@@ -18,7 +18,7 @@ BEGIN {
 
 use Exporter ();
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
-$VERSION   = '2.12_001';
+$VERSION   = '2.12_002';
 $VERSION   = eval $VERSION;
 @ISA       = qw(Exporter);
 @EXPORT    = qw(mkpath rmtree);
@@ -85,15 +85,15 @@ sub make_path {
 sub mkpath {
     my $old_style = !( @_ and __is_arg( $_[-1] ) );
 
-    my $arg;
+    my $data;
     my $paths;
 
     if ($old_style) {
         my ( $verbose, $mode );
         ( $paths, $verbose, $mode ) = @_;
         $paths = [$paths] unless UNIVERSAL::isa( $paths, 'ARRAY' );
-        $arg->{verbose} = $verbose;
-        $arg->{mode} = defined $mode ? $mode : oct '777';
+        $data->{verbose} = $verbose;
+        $data->{mode} = defined $mode ? $mode : oct '777';
     }
     else {
         my %args_permitted = map { $_ => 1 } ( qw|
@@ -108,50 +108,55 @@ sub mkpath {
             verbose
         | );
         my @bad_args = ();
-        $arg = pop @_;
+        my $arg = pop @_;
         for my $k (sort keys %{$arg}) {
-            push @bad_args, $k unless $args_permitted{$k};
+            if (! $args_permitted{$k}) {
+                push @bad_args, $k;
+            }
+            else {
+                $data->{$k} = $arg->{$k};
+            }
         }
         _carp("Unrecognized option(s) passed to make_path(): @bad_args")
             if @bad_args;
-        $arg->{mode} = delete $arg->{mask} if exists $arg->{mask};
-        $arg->{mode} = oct '777' unless exists $arg->{mode};
-        ${ $arg->{error} } = [] if exists $arg->{error};
-        $arg->{owner} = delete $arg->{user} if exists $arg->{user};
-        $arg->{owner} = delete $arg->{uid}  if exists $arg->{uid};
-        if ( exists $arg->{owner} and $arg->{owner} =~ /\D/ ) {
-            my $uid = ( getpwnam $arg->{owner} )[2];
+        $data->{mode} = delete $data->{mask} if exists $data->{mask};
+        $data->{mode} = oct '777' unless exists $data->{mode};
+        ${ $data->{error} } = [] if exists $data->{error};
+        $data->{owner} = delete $data->{user} if exists $data->{user};
+        $data->{owner} = delete $data->{uid}  if exists $data->{uid};
+        if ( exists $data->{owner} and $data->{owner} =~ /\D/ ) {
+            my $uid = ( getpwnam $data->{owner} )[2];
             if ( defined $uid ) {
-                $arg->{owner} = $uid;
+                $data->{owner} = $uid;
             }
             else {
-                _error( $arg,
-"unable to map $arg->{owner} to a uid, ownership not changed"
-                );
-                delete $arg->{owner};
+                _error( $data,
+                        "unable to map $data->{owner} to a uid, ownership not changed"
+                      );
+                delete $data->{owner};
             }
         }
-        if ( exists $arg->{group} and $arg->{group} =~ /\D/ ) {
-            my $gid = ( getgrnam $arg->{group} )[2];
+        if ( exists $data->{group} and $data->{group} =~ /\D/ ) {
+            my $gid = ( getgrnam $data->{group} )[2];
             if ( defined $gid ) {
-                $arg->{group} = $gid;
+                $data->{group} = $gid;
             }
             else {
-                _error( $arg,
-"unable to map $arg->{group} to a gid, group ownership not changed"
+                _error( $data,
+                        "unable to map $data->{group} to a gid, group ownership not changed"
                 );
-                delete $arg->{group};
+                delete $data->{group};
             }
         }
-        if ( exists $arg->{owner} and not exists $arg->{group} ) {
-            $arg->{group} = -1;    # chown will leave group unchanged
+        if ( exists $data->{owner} and not exists $data->{group} ) {
+            $data->{group} = -1;    # chown will leave group unchanged
         }
-        if ( exists $arg->{group} and not exists $arg->{owner} ) {
-            $arg->{owner} = -1;    # chown will leave owner unchanged
+        if ( exists $data->{group} and not exists $data->{owner} ) {
+            $data->{owner} = -1;    # chown will leave owner unchanged
         }
         $paths = [@_];
     }
-    return _mkpath( $arg, $paths );
+    return _mkpath( $data, $paths );
 }
 
 sub _mkpath {
@@ -238,14 +243,14 @@ sub _is_subdir {
 sub rmtree {
     my $old_style = !( @_ and __is_arg( $_[-1] ) );
 
-    my $arg;
+    my ($arg, $data);
     my $paths;
 
     if ($old_style) {
         my ( $verbose, $safe );
         ( $paths, $verbose, $safe ) = @_;
-        $arg->{verbose} = $verbose;
-        $arg->{safe} = defined $safe ? $safe : 0;
+        $data->{verbose} = $verbose;
+        $data->{safe} = defined $safe ? $safe : 0;
 
         if ( defined($paths) and length($paths) ) {
             $paths = [$paths] unless UNIVERSAL::isa( $paths, 'ARRAY' );
@@ -264,38 +269,43 @@ sub rmtree {
             verbose
         | );
         my @bad_args = ();
-        $arg = pop @_;
+        my $arg = pop @_;
         for my $k (sort keys %{$arg}) {
-            push @bad_args, $k unless $args_permitted{$k};
+            if (! $args_permitted{$k}) {
+                push @bad_args, $k;
+            }
+            else {
+                $data->{$k} = $arg->{$k};
+            }
         }
         _carp("Unrecognized option(s) passed to remove_tree(): @bad_args")
             if @bad_args;
-        ${ $arg->{error} }  = [] if exists $arg->{error};
-        ${ $arg->{result} } = [] if exists $arg->{result};
+        ${ $data->{error} }  = [] if exists $data->{error};
+        ${ $data->{result} } = [] if exists $data->{result};
         $paths = [@_];
     }
 
-    $arg->{prefix} = '';
-    $arg->{depth}  = 0;
+    $data->{prefix} = '';
+    $data->{depth}  = 0;
 
     my @clean_path;
-    $arg->{cwd} = getcwd() or do {
-        _error( $arg, "cannot fetch initial working directory" );
+    $data->{cwd} = getcwd() or do {
+        _error( $data, "cannot fetch initial working directory" );
         return 0;
     };
-    for ( $arg->{cwd} ) { /\A(.*)\Z/s; $_ = $1 }    # untaint
+    for ( $data->{cwd} ) { /\A(.*)\Z/s; $_ = $1 }    # untaint
 
     for my $p (@$paths) {
 
         # need to fixup case and map \ to / on Windows
         my $ortho_root = _IS_MSWIN32 ? _slash_lc($p) : $p;
         my $ortho_cwd =
-          _IS_MSWIN32 ? _slash_lc( $arg->{cwd} ) : $arg->{cwd};
+          _IS_MSWIN32 ? _slash_lc( $data->{cwd} ) : $data->{cwd};
         my $ortho_root_length = length($ortho_root);
         $ortho_root_length-- if _IS_VMS;   # don't compare '.' with ']'
         if ( $ortho_root_length && _is_subdir( $ortho_root, $ortho_cwd ) ) {
             local $! = 0;
-            _error( $arg, "cannot remove path when cwd is $arg->{cwd}", $p );
+            _error( $data, "cannot remove path when cwd is $data->{cwd}", $p );
             next;
         }
 
@@ -312,12 +322,12 @@ sub rmtree {
         push @clean_path, $p;
     }
 
-    @{$arg}{qw(device inode perm)} = ( lstat $arg->{cwd} )[ 0, 1 ] or do {
-        _error( $arg, "cannot stat initial working directory", $arg->{cwd} );
+    @{$data}{qw(device inode)} = ( lstat $data->{cwd} )[ 0, 1 ] or do {
+        _error( $data, "cannot stat initial working directory", $data->{cwd} );
         return 0;
     };
 
-    return _rmtree( $arg, \@clean_path );
+    return _rmtree( $data, \@clean_path );
 }
 
 sub _rmtree {
@@ -610,7 +620,7 @@ This document describes version 2.12 of File::Path.
 
 =head1 DESCRIPTION
 
-This module provide a convenient way to create directories of
+This module provides a convenient way to create directories of
 arbitrary depth and to delete an entire directory subtree from the
 filesystem.
 
@@ -623,7 +633,7 @@ The following functions are provided:
 =item make_path( $dir1, $dir2, ...., \%opts )
 
 The C<make_path> function creates the given directories if they don't
-exists before, much like the Unix command C<mkdir -p>.
+exist before, much like the Unix command C<mkdir -p>.
 
 The function accepts a list of directories to be created. Its
 behaviour may be tuned by an optional hashref appearing as the last
@@ -639,7 +649,7 @@ The following keys are recognised in the option hash:
 =item mode => $num
 
 The numeric permissions mode to apply to each created directory
-(defaults to 0777), to be modified by the current C<umask>. If the
+(defaults to C<0777>), to be modified by the current C<umask>. If the
 directory already exists (and thus does not need to be created),
 the permissions will not be modified.
 
@@ -675,9 +685,9 @@ in an C<eval> block.
 =item uid => $owner
 
 If present, will cause any created directory to be owned by C<$owner>.
-If the value is numeric, it will be interpreted as a uid, otherwise
-as username is assumed. An error will be issued if the username cannot be
-mapped to a uid, or the uid does not exist, or the process lacks the
+If the value is numeric, it will be interpreted as a uid, otherwise as
+username is assumed. An error will be issued if the username cannot be
+mapped to a uid, the uid does not exist or the process lacks the
 privileges to change ownership.
 
 Ownership of directories that already exist will not be changed.
@@ -686,11 +696,11 @@ C<user> and C<uid> are aliases of C<owner>.
 
 =item group => $group
 
-If present, will cause any created directory to be owned by the group C<$group>.
-If the value is numeric, it will be interpreted as a gid, otherwise
-as group name is assumed. An error will be issued if the group name cannot be
-mapped to a gid, or the gid does not exist, or the process lacks the
-privileges to change group ownership.
+If present, will cause any created directory to be owned by the group
+C<$group>.  If the value is numeric, it will be interpreted as a gid,
+otherwise as group name is assumed. An error will be issued if the
+group name cannot be mapped to a gid, the gid does not exist or the
+process lacks the privileges to change group ownership.
 
 Group ownership of directories that already exist will not be changed.
 
@@ -706,9 +716,10 @@ Group ownership of directories that already exist will not be changed.
 
 =item mkpath( $dir1, $dir2,..., \%opt )
 
-The mkpath() function provide the legacy interface of make_path() with
-a different interpretation of the arguments passed.  The behaviour and
-return value of the function is otherwise identical to make_path().
+The C<mkpath()> function provide the legacy interface of
+C<make_path()> with a different interpretation of the arguments
+passed.  The behaviour and return value of the function is otherwise
+identical to C<make_path()>.
 
 =item remove_tree( $dir1, $dir2, .... )
 
@@ -725,7 +736,7 @@ removed. Its behaviour may be tuned by an optional hashref
 appearing as the last parameter on the call.  If an empty string is
 passed to C<remove_tree>, an error will occur.
 
-The functions returns the number of files successfully deleted.
+The function returns the number of files successfully deleted.
 
 The following keys are recognised in the option hash:
 
@@ -791,10 +802,10 @@ of hand. This is the safest course of action.
 
 =item rmtree( $dir1, $dir2,..., \%opt )
 
-The rmtree() function provide the legacy interface of remove_tree()
-with a different interpretation of the arguments passed. The behaviour
-and return value of the function is otherwise identical to
-remove_tree().
+The C<rmtree()> function provide the legacy interface of
+C<remove_tree()> with a different interpretation of the arguments
+passed. The behaviour and return value of the function is otherwise
+identical to C<remove_tree()>.
 
 =back
 
@@ -813,9 +824,9 @@ C<make_path> or C<remove_tree>, you should take additional precautions.
 
 =back
 
-If C<make_path> or C<remove_tree> encounter an error, a diagnostic
+If C<make_path> or C<remove_tree> encounters an error, a diagnostic
 message will be printed to C<STDERR> via C<carp> (for non-fatal
-errors), or via C<croak> (for fatal errors).
+errors) or via C<croak> (for fatal errors).
 
 If this behaviour is not desirable, the C<error> attribute may be
 used to hold a reference to a variable, which will be used to store
@@ -882,7 +893,7 @@ to at least 2.08 in order to avoid surprises.
 
 =head3 SECURITY CONSIDERATIONS
 
-There were race conditions 1.x implementations of File::Path's
+There were race conditions in the 1.x implementations of File::Path's
 C<rmtree> function (although sometimes patched depending on the OS
 distribution or platform). The 2.0 version contains code to avoid the
 problem mentioned in CVE-2002-0435.
@@ -907,7 +918,7 @@ can always be trapped with C<eval>, but it's not a good idea. Under
 the circumstances, dying is the best thing to do).
 
 SEVERE errors may be trapped using the modern interface. If the
-they are not trapped, or the old interface is used, such an error
+they are not trapped, or if the old interface is used, such an error
 will cause the program will halt.
 
 All other errors may be trapped using the modern interface, otherwise
@@ -918,7 +929,7 @@ they will be C<carp>ed about. Program execution will not be halted.
 =item mkdir [path]: [errmsg] (SEVERE)
 
 C<make_path> was unable to create the path. Probably some sort of
-permissions error at the point of departure, or insufficient resources
+permissions error at the point of departure or insufficient resources
 (such as free inodes on Unix).
 
 =item No root path(s) specified
@@ -997,7 +1008,7 @@ halts to avoid a race condition from occurring.
 
 =item cannot stat prior working directory [dir]: [errmsg], aborting. (FATAL)
 
-C<remove_tree> was unable to stat the parent directory after have returned
+C<remove_tree> was unable to stat the parent directory after having returned
 from the child. Since there is no way of knowing if we returned to
 where we think we should be (by comparing device and inode) the only
 way out is to C<croak>.
@@ -1017,9 +1028,9 @@ execution continues, but the directory may possibly not be deleted.
 
 =item cannot remove directory [dir]: [errmsg]
 
-C<remove_tree> attempted to remove a directory, but failed. This may because
+C<remove_tree> attempted to remove a directory, but failed. This may be because
 some objects that were unable to be removed remain in the directory, or
-a permissions issue. The directory will be left behind.
+it could be a permissions issue. The directory will be left behind.
 
 =item cannot restore permissions of [dir] to [0nnn]: [errmsg]
 
@@ -1087,15 +1098,16 @@ to examining directory trees.
 
 The following describes F<File::Path> limitations and how to report bugs.
 
-=head2 MULTITHREAD APPLICATIONS
+=head2 MULTITHREADED APPLICATIONS
 
-F<File::Path> B<rmtree> and B<remove_tree> will not work with multithreaded
-applications due to its use of B<chdir>.  At this time, no warning or error
-results and you will certainly encounter unexpected results.
+F<File::Path> C<rmtree> and C<remove_tree> will not work with
+multithreaded applications due to its use of C<chdir>.  At this time,
+no warning or error is generated in this situation.  You will
+certainly encounter unexpected results.
 
 The implementation that surfaces this limitation will not be changed. See the
-F<File::Path::Tiny> module for functionality similar to F<File::Path> but does
-not B<chdir>.
+F<File::Path::Tiny> module for functionality similar to F<File::Path> but which does
+not C<chdir>.
 
 =head2 NFS Mount Points
 
@@ -1148,6 +1160,8 @@ Contributors to File::Path, in alphabetical order.
 
 =item <F<bulkdd@cpan.org>>
 
+=item Charlie Gonzalez <F<itcharlie@cpan.org>>
+
 =item Craig A. Berry <F<craigberry@mac.com>>
 
 =item Richard Elberger <F<riche@cpan.org>>
@@ -1165,7 +1179,7 @@ Contributors to File::Path, in alphabetical order.
 =head1 COPYRIGHT
 
 This module is copyright (C) Charles Bailey, Tim Bunce, David Landgren,
-James Keenan, and Richard Elberger 1995-2015. All rights reserved.
+James Keenan and Richard Elberger 1995-2017. All rights reserved.
 
 =head1 LICENSE
 
